//******************************************************************************
//*
//*  df_watermark.jsx
//*           for Adobe Photoshop CSx
//*
//*  Revision 1.0
//*
//*  Copyright (c) 2013 dwarf_foto. All rights reserved.
//*
//*  http://whitemania.jp/dwarf_foto/
//*
//*  History:
//*  Rev    Date            Author      Comment
//*  ---------------------------------------------------------------------------
//*   1.0   17-Feb-2013     dwarf       First Release 
//*
//******************************************************************************
#target photoshop
//@include "df_watermark/df_watermark_settings.jsx";


//******************************************************************************
//***  Photoshopメニュー表示設定 
//******************************************************************************
/*
<javascriptresource>
<name>ウォーターマークの挿入...</name>
<category>zzz_dwarf_foto_category</category>
</javascriptresource>
*/


//------------------------------------------------------------------------------


//******************************************************************************
//***  定数 
//******************************************************************************
//------------------------------------------------------------------------------
//  ウォーターマーク位置フラグ 
//------------------------------------------------------------------------------
// 位置フラグ 
const DEF_NONE                  = 0;        // なし 
const DEF_WHITE_TOP_LEFT        = 0x0001;   // 左上 (白) 
const DEF_WHITE_TOP_RIGHT       = 0x0002;   // 右上 (白) 
const DEF_WHITE_BTM_LEFT        = 0x0004;   // 左下 (白) 
const DEF_WHITE_BTM_RIGHT       = 0x0008;   // 右下 (白) 
const DEF_BLACK_TOP_LEFT        = 0x0100;   // 左上 (黒) 
const DEF_BLACK_TOP_RIGHT       = 0x0200;   // 右上 (黒) 
const DEF_BLACK_BTM_LEFT        = 0x0400;   // 左下 (黒) 
const DEF_BLACK_BTM_RIGHT       = 0x0800;   // 右下 (黒) 
// マスク 
const DEF_WHITE_MASK            = 0x00FF;   // 白 
const DEF_BLACK_MASK            = 0xFF00;   // 黒 

//------------------------------------------------------------------------------
//  出力タイプ 
//------------------------------------------------------------------------------
const DEF_OUTPUT_FILE           = 1;        // ファイルへ出力 
const DEF_OUTPUT_LAYER          = 2;        // アクティブドキュメントへレイヤー追加 

//------------------------------------------------------------------------------
//  出力ファイルタイプ 
//------------------------------------------------------------------------------
const DEF_OUTPUT_JPEG           = 1;        // JPEG 
const DEF_OUTPUT_TIFF           = 2;        // TIFF 

//------------------------------------------------------------------------------
//  ウォーターマークリサイズフラグ 
//------------------------------------------------------------------------------
const DEF_WM_SCALE_NONE         = 0;        // なし 
const DEF_WM_SCALE_RESIZE       = 0x0001;   // リサイズする 
const DEF_WM_SCALE_OFFSET       = 0x0002;   // オフセットをスケーリングする 
const DEF_WM_SCALE_BASE_LONG    = 0x0004;   // 長辺を基準とする 


//------------------------------------------------------------------------------


//******************************************************************************
//***  ユーティリティ関数 
//******************************************************************************

//------------------------------------------------------------------------------
//  スクリプトフルパス名取得 
//------------------------------------------------------------------------------
function getScriptFullPathName()
{
    try
    {
        ERROR;
    }
    catch(e)
    {
        return e.fileName;
    }
}

//------------------------------------------------------------------------------
//  スクリプトフォルダ名取得 
//------------------------------------------------------------------------------
function getScriptPathName()
{
    return File(getScriptFullPathName()).parent;
}

//------------------------------------------------------------------------------
//  スクリプトファイル名取得 
//------------------------------------------------------------------------------
function getScriptFileName()
{
    return File(getScriptFullPathName()).name.split(".")[0];
}

//------------------------------------------------------------------------------
//  フォルダ名にデリミタを付ける 
//------------------------------------------------------------------------------
function addDelimiterPathName(path)
{
    if (path[path.length - 1] != '\\')
    {
        path = path + '\\';
    }
    return path;
}


//------------------------------------------------------------------------------


//******************************************************************************
//***  メイン処理 
//******************************************************************************

//------------------------------------------------------------------------------
//  オプション設定関数呼び出し 
//------------------------------------------------------------------------------
initUserSettings();


//------------------------------------------------------------------------------
//  出力ファイル名生成 
//------------------------------------------------------------------------------
var strExt = (OUTPUT_FILE_TYPE == DEF_OUTPUT_TIFF) ? ".tif" : ".jpg";
var strOutputFName = activeDocument.name.split(".")[0] + strExt;

//------------------------------------------------------------------------------
//  選択ダイアログ処理 
//------------------------------------------------------------------------------
if (WATERMARK_POSITION == DEF_NONE)
{
    //--------------------------------------------------------------------------
    //  ダイアログ生成 
    //--------------------------------------------------------------------------
    var WINDOW_WIDTH  = 280
    var WINDOW_HEIGHT = 215
    var WINDOW_HEIGHT_ADD = 60;
    var dlgSelect = new Window("dialog", "ウォーターマークの挿入");
    dlgSelect.size = [WINDOW_WIDTH,WINDOW_HEIGHT+WINDOW_HEIGHT_ADD];
    dlgSelect.center();

    // ウォーターマーク位置選択ボタン 
    var BUTTON_LEFT_W = 10;
    var BUTTON_TOP_W  = 35;
    var BUTTON_LEFT_B = BUTTON_LEFT_W + 140;
    var BUTTON_TOP_B  = BUTTON_TOP_W;
    var BUTTON_WIDTH  = 60;
    var BUTTON_HEIGHT = 40;

    var stWatermarkW = dlgSelect.add("statictext", [BUTTON_LEFT_W,10,BUTTON_LEFT_W+100,30], "白ロゴ");
    var stWatermarkB = dlgSelect.add("statictext", [BUTTON_LEFT_B,10,BUTTON_LEFT_B+100,30], "黒ロゴ");

    var btnWatermarkWTL = dlgSelect.add("button", [BUTTON_LEFT_W,              BUTTON_TOP_W,               BUTTON_LEFT_W+BUTTON_WIDTH,   BUTTON_TOP_W+BUTTON_HEIGHT],   "左上");
    var btnWatermarkWTR = dlgSelect.add("button", [BUTTON_LEFT_W+BUTTON_WIDTH, BUTTON_TOP_W,               BUTTON_LEFT_W+BUTTON_WIDTH*2, BUTTON_TOP_W+BUTTON_HEIGHT],   "右上");
    var btnWatermarkWBL = dlgSelect.add("button", [BUTTON_LEFT_W,              BUTTON_TOP_W+BUTTON_HEIGHT, BUTTON_LEFT_W+BUTTON_WIDTH,   BUTTON_TOP_W+BUTTON_HEIGHT*2], "左下");
    var btnWatermarkWBR = dlgSelect.add("button", [BUTTON_LEFT_W+BUTTON_WIDTH, BUTTON_TOP_W+BUTTON_HEIGHT, BUTTON_LEFT_W+BUTTON_WIDTH*2, BUTTON_TOP_W+BUTTON_HEIGHT*2], "右下");

    var btnWatermarkBTL = dlgSelect.add("button", [BUTTON_LEFT_B,              BUTTON_TOP_B,               BUTTON_LEFT_B+BUTTON_WIDTH,   BUTTON_TOP_B+BUTTON_HEIGHT],   "左上");
    var btnWatermarkBTR = dlgSelect.add("button", [BUTTON_LEFT_B+BUTTON_WIDTH, BUTTON_TOP_B,               BUTTON_LEFT_B+BUTTON_WIDTH*2, BUTTON_TOP_B+BUTTON_HEIGHT],   "右上");
    var btnWatermarkBBL = dlgSelect.add("button", [BUTTON_LEFT_B,              BUTTON_TOP_B+BUTTON_HEIGHT, BUTTON_LEFT_B+BUTTON_WIDTH,   BUTTON_TOP_B+BUTTON_HEIGHT*2], "左下");
    var btnWatermarkBBR = dlgSelect.add("button", [BUTTON_LEFT_B+BUTTON_WIDTH, BUTTON_TOP_B+BUTTON_HEIGHT, BUTTON_LEFT_B+BUTTON_WIDTH*2, BUTTON_TOP_B+BUTTON_HEIGHT*2], "右下");

    // キャンセルボタン 
    var btnCancel = dlgSelect.add("button", [100,138,170,138+25], "Cancel", {name: "cancel"});

    // 出力チェック 
    var OUTPUTTYPE_TOP = 180;
    var chkOutputType = dlgSelect.add("checkbox", [10,OUTPUTTYPE_TOP,270,OUTPUTTYPE_TOP+20], "ウォーターマーク挿入画像をファイルに出力する");
    chkOutputType.helpTip = "チェック解除でウォーターマークレイヤーを追加"
    chkOutputType.value = (OUTPUT_TYPE == DEF_OUTPUT_FILE);

    // ファイルタイプ 
    var FILETYPE_TOP = 210;
    var stFileType  = dlgSelect.add("statictext", [30,FILETYPE_TOP+2,130,FILETYPE_TOP+19], "出力ファイルタイプ");
    var ddlFileType = dlgSelect.add("dropdownlist", [130,FILETYPE_TOP,270,FILETYPE_TOP+20], ["TIFF", "JPEG"]);
    ddlFileType.selection = (OUTPUT_FILE_TYPE == DEF_OUTPUT_TIFF) ? 0 : 1;

    // 出力ファイル名テキストボックス 
    var OUTFNAME_TOP = 240;
    var stOutFName = dlgSelect.add("statictext", [30,OUTFNAME_TOP+2,130,OUTFNAME_TOP+19], "出力ファイル名");
    var tbOutFName = dlgSelect.add("edittext", [130,OUTFNAME_TOP,270,OUTFNAME_TOP+20], strOutputFName);


    //--------------------------------------------------------------------------
    //  ダイアログイベント処理 
    //--------------------------------------------------------------------------
    btnWatermarkWTL.onClick = function()        // 白左上ボタンクリック 
    {
        WATERMARK_POSITION = DEF_WHITE_TOP_LEFT;
        dlgSelect.close();
    }
    btnWatermarkWTR.onClick = function()        // 白右上ボタンクリック 
    {
        WATERMARK_POSITION = DEF_WHITE_TOP_RIGHT;
        dlgSelect.close();
    }
    btnWatermarkWBL.onClick = function()        // 白左下ボタンクリック 
    {
        WATERMARK_POSITION = DEF_WHITE_BTM_LEFT;
        dlgSelect.close();
    }
    btnWatermarkWBR.onClick = function()        // 白右下ボタンクリック 
    {
        WATERMARK_POSITION = DEF_WHITE_BTM_RIGHT;
        dlgSelect.close();
    }

    btnWatermarkBTL.onClick = function()        // 黒左上ボタンクリック 
    {
        WATERMARK_POSITION = DEF_BLACK_TOP_LEFT;
        dlgSelect.close();
    }
    btnWatermarkBTR.onClick = function()        // 黒右上ボタンクリック 
    {
        WATERMARK_POSITION = DEF_BLACK_TOP_RIGHT;
        dlgSelect.close();
    }
    btnWatermarkBBL.onClick = function()        // 黒左下ボタンクリック 
    {
        WATERMARK_POSITION = DEF_BLACK_BTM_LEFT;
        dlgSelect.close();
    }
    btnWatermarkBBR.onClick = function()        // 黒右下ボタンクリック 
    {
        WATERMARK_POSITION = DEF_BLACK_BTM_RIGHT;
        dlgSelect.close();
    }

    chkOutputType.onClick = function()          // チェックボックス切り替え 
    {
        var numState = chkOutputType.value;

        // ファイル出力オプションを非表示に 
        stFileType.visible  = numState;
        ddlFileType.visible = numState;
        stOutFName.visible  = numState;
        tbOutFName.visible  = numState;

        // ダイアログサイズ変更 
        var numOffset = (numState) ? WINDOW_HEIGHT_ADD : 0;
        dlgSelect.size = [WINDOW_WIDTH,WINDOW_HEIGHT+numOffset];
    }
    chkOutputType.onClick();    // 一度オプションを反映させておく 

    ddlFileType.onChange = function()           // ファイルタイプ選択変更 
    {
        // 出力ファイル名バリデーション処理 
        var strSplit = tbOutFName.text.split(".");
        if ((strSplit.length > 1) && 
            ( ((ddlFileType.selection == 1) && (strSplit[1].toLowerCase() == "tif")) || 
              ((ddlFileType.selection == 0) && (strSplit[1].toLowerCase() == "jpg")) ))
        {
            strExt = (ddlFileType.selection == 0) ? ".tif" : ".jpg";
            tbOutFName.text = tbOutFName.text.split(".")[0] + strExt;
        }
    }

    btnCancel.onClick = function()              // キャンセルボタンクリック 
    {
        exit();
        dlgSelect.close();
    }

    dlgSelect.show();       // ダイアログ表示
}

//------------------------------------------------------------------------------
// ウォーターマーク出力処理 
//------------------------------------------------------------------------------
if (WATERMARK_POSITION != DEF_NONE)
{
    //--------------------------------------------------------------------------
    // ダイアログでのオプション設定処理 
    //--------------------------------------------------------------------------
    OUTPUT_TYPE = (chkOutputType.value) ? DEF_OUTPUT_FILE : DEF_OUTPUT_LAYER;
    OUTPUT_FILE_TYPE = (ddlFileType.selection == 0) ? DEF_OUTPUT_TIFF : DEF_OUTPUT_JPEG;

    //--------------------------------------------------------------------------
    // ファイル入力処理＆下ごしらえ 
    //--------------------------------------------------------------------------
    var objOriginal = activeDocument;           // 現在開いているドキュメントを取得 
    // ファイル出力の場合ウォーターマーク用作業用ドキュメントとして、現在開いているドキュメントを複製。 
    var objWork = (OUTPUT_TYPE == DEF_OUTPUT_FILE) ? objOriginal.duplicate() : objOriginal;

    preferences.rulerUnits = Units.PIXELS;

    // 画像サイズを取得 
    var numWidth  = objWork.width.value;
    var numHeight = objWork.height.value;

    // スケーリング計算 
    var numScaling = 1;
    if (WATERMARK_SCALE_SIZE > 0)
    {
        if (WATERMARK_SCALE_OPTION & DEF_WM_SCALE_BASE_LONG)    numScaling = ((numWidth > numHeight) ? numWidth : numHeight) / WATERMARK_SCALE_SIZE;
        else                                                    numScaling = ((numWidth < numHeight) ? numWidth : numHeight) / WATERMARK_SCALE_SIZE;
    }

    // ウォーターマーク画像のファイルを開く 
    var strWatermarkFName;
    var objWatermark;
    var objFile;
    try
    {
        if (WATERMARK_POSITION & DEF_WHITE_MASK)    strWatermarkFName = WATERMARK_PATH_WHITE;   // 白ロゴ 
        else                                        strWatermarkFName = WATERMARK_PATH_BLACK;   // 黒ロゴ 
        objFile = new File(strWatermarkFName);
        objWatermark = open(objFile);
    }
    catch (e)
    {
        // ファイルを開けなかったらスクリプトフォルダを検索 
        if (WATERMARK_POSITION & DEF_WHITE_MASK)    strWatermarkFName = addDelimiterPathName(getScriptPathName()) + WATERMARK_PATH_WHITE;   // 白ロゴ 
        else                                        strWatermarkFName = addDelimiterPathName(getScriptPathName()) + WATERMARK_PATH_BLACK;   // 黒ロゴ 
        objFile = new File(strWatermarkFName);
        objWatermark = open(objFile);
    }

    // ウォーターマークのスケーリング 
    if (WATERMARK_SCALE_OPTION & DEF_WM_SCALE_RESIZE)
    {
        var numResizeWidth  = 0;
        var numResizeHeight = 0;

        if ( (((WATERMARK_SCALE_OPTION & DEF_WM_SCALE_BASE_LONG) == 0) && (objWatermark.width.value < objWatermark.height.value)) || 
             ( (WATERMARK_SCALE_OPTION & DEF_WM_SCALE_BASE_LONG)       && (objWatermark.width.value > objWatermark.height.value)) )
        {
            numResizeWidth  = Math.floor(objWatermark.width.value * numScaling);
            numResizeHeight = Math.floor((numResizeWidth / objWatermark.width.value) * objWatermark.height.value);
        }
        else
        {
            numResizeHeight = Math.floor(objWatermark.height.value * numScaling);
            numResizeWidth  = Math.floor((numResizeHeight / objWatermark.height.value) * objWatermark.width.value);
        }
        objWatermark.resizeImage(numResizeWidth, numResizeHeight, objWatermark.resolution, ResampleMethod.BICUBICSHARPER);
    }

    // ウォーターマークの画像サイズを取得 
    var numWatermarkWidth  = objWatermark.width.value;
    var numWatermarkHeight = objWatermark.height.value;

    // ウォーターマークのすべてのレイヤーをワークドキュメントへコピー 
    var numLayerWatermark = objWatermark.layers.length;
    for (i = 0; i < numLayerWatermark; i++)
    {
        objWatermark.layers[i].duplicate(objWork);
    }

    // ウォーターマーク画像を閉じる 
    objWatermark.close(SaveOptions.DONOTSAVECHANGES);
    objFile.close();


    //--------------------------------------------------------------------------
    // ウォーターマーク位置移動処理 
    //--------------------------------------------------------------------------
    // ウォーターマークの画像位置を計算 
    var numWatermarkLeft = 0;
    var numWatermarkTop  = 0;
    var numOffsetX = WATERMARK_MARGIN_X;
    var numOffsetY = WATERMARK_MARGIN_Y;

    // オフセットのスケーリング 
    if (WATERMARK_SCALE_OPTION & DEF_WM_SCALE_OFFSET)
    {
        numOffsetX = WATERMARK_MARGIN_X * numScaling;
        numOffsetY = WATERMARK_MARGIN_Y * numScaling;
    }

    switch (WATERMARK_POSITION)
    {
        case DEF_WHITE_TOP_LEFT:    // 左上 
        case DEF_BLACK_TOP_LEFT:
            numWatermarkLeft = numOffsetX;
            numWatermarkTop  = numOffsetY;
            break;

        case DEF_WHITE_TOP_RIGHT:   // 右上 
        case DEF_BLACK_TOP_RIGHT:
            numWatermarkLeft = numWidth - numWatermarkWidth - numOffsetX;
            numWatermarkTop  = numOffsetY;
            break;

        case DEF_WHITE_BTM_LEFT:    // 左下 
        case DEF_BLACK_BTM_LEFT:
            numWatermarkLeft = numOffsetX;
            numWatermarkTop  = numHeight - numWatermarkHeight - numOffsetY;
            break;

        case DEF_WHITE_BTM_RIGHT:   // 右下 
        case DEF_BLACK_BTM_RIGHT:
            numWatermarkLeft = numWidth - numWatermarkWidth - numOffsetX;
            numWatermarkTop  = numHeight - numWatermarkHeight - numOffsetY;
            break;
    }

    // ウォーターマークレイヤーの移動 (コピーされたレイヤーは0番から) 
    var numLayer = objWork.layers.length;
    for (i = 0; i < numLayerWatermark; i++)
    {
        objWork.layers[i].translate(numWatermarkLeft, numWatermarkTop);
    }


    //--------------------------------------------------------------------------
    // ファイル出力処理 
    //--------------------------------------------------------------------------
    if (OUTPUT_TYPE == DEF_OUTPUT_FILE)
    {
        // 画像出力フォルダ生成 
        var strOutputFolder = OUTPUT_FOLDER_NAME;
        if ((strOutputFolder.length < 3) || (strOutputFolder[1] != ':') || (strOutputFolder[2] != '\\'))    // 相対パスチェック 
        {
            strOutputFolder = addDelimiterPathName(objOriginal.path) + OUTPUT_FOLDER_NAME;
        }
        var objFolder = new Folder(strOutputFolder);
        objFolder.create();

        // ダイアログでファイル名が指定されていればそれを使う。 
        try
        {
            if (tbOutFName.text != "")
            {
                strOutputFName = tbOutFName.text;
            }
        }
        catch (e)
        {   // Do nothing 
        }

        objFile = new File(addDelimiterPathName(strOutputFolder) + strOutputFName);

        var objOption;
        if (OUTPUT_FILE_TYPE == DEF_OUTPUT_TIFF)
        {
            // TIFFファイル出力 (LZW圧縮、レイヤー破棄) 
            objOption = new TiffSaveOptions();
            objOption.alphaChannels     = false;
            objOption.annotations       = false;
            objOption.byteOrder         = ByteOrder.IBM;
            objOption.embedColorProfile = true;
            objOption.imageCompression  = TIFFEncoding.TIFFLZW;
            objOption.jpegQuality       = 11;
            objOption.layerCompression  = LayerCompression.RLE;
            objOption.layers            = false;
            objOption.saveImagePyramid  = false;
            objOption.spotColors        = false;
            objOption.transparency      = false;
        }
        else
        {
            // JPEGファイル出力 
            objOption = new JPEGSaveOptions();
            objOption.embedColorProfile = true;
            objOption.quality           = 8;
            objOption.formatOptions     = FormatOptions.STANDARDBASELINE;
            objOption.matte             = MatteType.NONE;
        }
        objWork.saveAs(objFile, objOption, true, Extension.LOWERCASE);  // ウォーターマーク差し込み画像を出力 

        // ファイル後処理 
        objFile.close();
        objWork.close(SaveOptions.DONOTSAVECHANGES);    // ワークドキュメントを保存せず閉じる 
    }
}

//******************************************************************************
//***  End of file 
//******************************************************************************
